#define R_EQUATORIAL 6378137 /* metres */
#define R_POLAR 6356752.3142  /* metres */
#define NUM_DIGITS_ENTERS_LATITUD 2
#define NUM_DIGITS_DECIMALS_LATITUD 10
#define NUM_DIGITS_ENTERS_LONGITUD 3
#define NUM_DIGITS_DECIMALS_LONGITUD 10
#define N_BATERIA 3

#define N_BUFFER 120 //150
#define M_BUFFER 30 //50
#define MEM_MAX 10 
/*Es guardaran 10 enviaments en memòria en el cas que no hi hagi cobertura 3G/GPRS. 
 Cadascun d'aquests enviaments ocupa 24 bytes. 
 
Alerta: la memòria SRAM és de 2 KB. En el cas que s'acabi, el programa pot quedar 
encallat, o l'Arduino pot reinicialitzar el codi, a partir de la funció setup().
Augmentar el nombre de crides a funcions, per tal d'evitar repetir codir, suposa també
una càrrega de memòria SRAM. Una de les causes que una petita modificació en el codi ocasioni
que el codi deixi funcionar correctament pot ser aquesta.
 */
#define ESTAT_ERROR 0
#define ESTAT_ENVIAT 1
#define ESTAT_REBENT 2
#define ESTAT_REQUEST 3
#define ESTAT_REINICIAT 2
#define ERROR 4


/******************************************************************************
 *******************************************************************************
 *********************** VALORS CONFIGURABLES **********************************
 *******************************************************************************
 *******************************************************************************/

/*Coordenades de la meta a Sant Feliu de Guíxols*/
#define LATITUD_META 41.7792625
#define LONGITUD_META 3.03776026 
#define DISTANCIA_META 50 
/*Es considera que s'ha arribat a la meta si el punt es
 troba una distància en metres respecte la meta igual o inferior al valor donat per
 DISTANCIA_META*/
#define DETECTAR_META 1 //Per detectar si s'ha arribat a la meta o no.
//#define DETECTAR_META 0 //Per no detectar si s'ha arribat a la meta o no.


/*Durada en minuts de l'interval de temps d'espera abans d'enviar les seguents dades,
 quan hi ha cobertura 3G/GPRS*/
#define MINUTS 10 

/*Durada en minuts de l'interval de temps d'espera abans d'enviar les seguents dades,
 quan no hi ha cobertura 3G/GPRS*/
#define MINUTS_NO_GPRS 10 //15 potser millor

char Equip[]="ARDUINO_147";
const char server[]="213.73.41.144";
const char port[]="80";

const char codi_PIN[]="6944";        // PIN de la targeta SIM
const char APN[]="gprs-service.com"; // Aquest és l'APN per Internet de la companyia telefònica Simyo
/******************************************************************************
 *******************************************************************************
 *********************** FINAL VALORS CONFIGURABLES ****************************
 *******************************************************************************
 *******************************************************************************/
struct registre{  
  double latitud; // En les plaques Arduino amb un microcontrolador ATmega, double no 
  // és realment double, és float, y ocupa 4 bytes.
  double longitud;
  char any[2];
  char mes[2];
  char dia[2];
  char hora[2];
  char minut[2];
  char segon[2];
  char bateria[N_BATERIA]; //3 bytes
  char enviat;
}; 	 //Ocupa 24 bytes
struct temps{
  char hora[2];
  char minut[2];
  char segon[2];
};

struct posicions{
  int signe_latitud;
  int latitud;
  int signe_longitud;
  int longitud;
  int IMSI;
  int bateria;
  int any;
  int mes;
  int dia;
  int hora;
  int minut;
  int segon;
};

const int led = 13;
const int onModulePin = 2; // El pin per engegar el mòdul, sense prémer el botó 

void switchModule(){
  /******************************************************************
   * Engega el mòdul
   *******************************************************************/
  digitalWrite(onModulePin,HIGH);
  delay(2000);
  digitalWrite(onModulePin,LOW);
}

void setup(){
  /******************************************************************
   *	S'executa una única vegada després que arranqui la placa Arduino.
   *******************************************************************/

  Serial.begin(115200);            // Velocitat en bauds de l'UART.
  delay(2000);
  pinMode(led, OUTPUT);
  pinMode(onModulePin, OUTPUT);

  reiniciar_modul();
}

void iniciar_modul(){
  /******************************************************************
   *  Engega i configura el mòdul introduint el PIN i definint
   *  el context PDP del socket.
   *******************************************************************/
  int k;
  switchModule();                    // Engega el mòdul.
  for (k=0;k<5;k++){
    delay(5000);
  } 
  Serial.flush();//__________
  delay(1000);
  Serial.print("AT+CPIN="); //Introdueix PIN targeta SIM
  Serial.println(codi_PIN);
  espera_OK(); 
  Serial.flush();
  delay(3000);
  Serial.print("AT+CGSOCKCONT=1,\"IP\",\""); //Defineix el context PDP del socket.
  Serial.print(APN);
  Serial.println("\"");
  espera_OK(); 
}

void reiniciar_modul(){
  /******************************************************************
   *  Si el mòdul SIM5218 està engegat i està en un estat que accepta 
   *  comandes AT, l'apaga. Després torna
   *  a engegar-lo i a configurar-lo: PIN i context PDP del socket.
   *******************************************************************/
  Serial.println("AT+CPOF"); 
  // Si el mòdul SIM5218 està engegat, l'apaga, sinó aquesta comanda no fa res.
  // En el cas que l'Arduino es reinicialitzi automàticament (a vegades ho fa) és imprescindible apagar el
  // módul i tornar-lo a engegar, perquè sinó la comanda AT+CPIN no s'executaria tal com s'espera. Com el mòdul
  // està engegat entendria que entres el PIN2, ja que el PIN ja està entrat. Com que el que estàs entrant per 
  // al PIN2 és el PIN 1, aleshores retorna que has entrat un pin erroni i no retorna OK.

  delay(5000);
  iniciar_modul();
}  
void reiniciar_modul_bloquejat(){
  /******************************************************************
   *  Si el mòdul SIM5218 està engegat i està en un estat que no accepta 
   *  comandes AT, l'apaga. Després torna
   *  a engegar-lo i a configurar-lo: PIN i context PDP del socket.
   *******************************************************************/

  switchModule(); //Aquesta funció també serveix per apagar el mòdul.
  delay(25000);  // Aquest temps probablement es pot reduir. No s'han fet proves en aquest sentit    
  reiniciar_modul();
} 

int guarda(struct registre reg_actual, struct registre reg_guardats[], int i_guardar, int max)
/*********************************************************************************************
 * Emmagatzema la informació recollida en reg_actual, en un vector de registres en la posició
 * i_guardar. El nombre de registres del vector de registres és max. 
 *
 *    Paràmetres d'entrada:
 *			regitre reg_actual: registre que es vol emmagatzemar en reg_guardats.
 *			reg_guardats: vector de registres.
 *			int i_guardar: posició del vector de registres on s'ha d'emmagatzemar el registre 
 *				reg_actual.
 *			int max: dimensió del vector de registres.
 *    Valor de retorn(int): posició on s'ha d'emmagatzemar el següent registre.
 **********************************************************************************************/
{
  i_guardar=i_guardar%max;
  reg_guardats[i_guardar].latitud=reg_actual.latitud;
  reg_guardats[i_guardar].longitud=reg_actual.longitud;
  copia_caracters(2, reg_actual.any, reg_guardats[i_guardar].any);
  copia_caracters(2, reg_actual.mes, reg_guardats[i_guardar].mes);
  copia_caracters(2, reg_actual.dia, reg_guardats[i_guardar].dia);
  copia_caracters(2, reg_actual.hora, reg_guardats[i_guardar].hora);
  copia_caracters(2, reg_actual.minut, reg_guardats[i_guardar].minut);
  copia_caracters(2, reg_actual.segon, reg_guardats[i_guardar].segon);
  copia_caracters(3, reg_actual.bateria, reg_guardats[i_guardar].bateria);	
  reg_guardats[i_guardar].enviat=0;
  i_guardar=(i_guardar+1)%max;
  return i_guardar;
}	

int detectar_meta(double radi, double latitud, double longitud, double latitud0, double longitud0){
  /************************************************************************************************
   * Donat un punt sobre la terra del qual es coneixen les seves coordenades de latitud i longitud, 
   *	i el radi de la terra en aquestes coordenades, es calcula la distància a una meta donada.
   *
   *	Paràmetres d'entrada:
   *		double radi: radi de la terra en el punt.
   *		double latitud: latitud del punt.
   *		double longitud: longitud del punt.
   *		double latitud0: latitud de la meta.
   *		double longitud0: longitud de la meta.
   *	Valor de retorn (int):
   *		0: si la distància és inferior o igual a DISTANCIA_META (metres).
   *		1: si la distància es més gran que 20 metres i inferior o igual a 1000 metres.
   *		2: si la distància és més gran que 1000 metres i inferior o igual a 5000 metres.
   *		3: si la distància és més gran que 5000 metres.
   *************************************************************************************************/
  double d;
  double a,b,a0,b0 ;
  a=latitud*M_PI/180;
  b=longitud*M_PI/180;
  a0=latitud0*M_PI/180;
  b0=longitud0*M_PI/180;
  d=radi*acos(sin(a)*sin(a0)+cos(a)*cos(a0)*cos(b-b0));
  printf ("dist %lf\n",d);
  if (d>5000) return 3; /*5000 metres*/
  else if (d>1000) return 2; /*1000 metres*/
  else if (d>DISTANCIA_META) return 1; 
  else return 0;
}  

double radi_elipse (double angle, double a, double b){
  /*************************************************************************************
   *	Donada una el·lipse de semieix major a i semieix menor b, retorna el radi d'un punt que 
   *	es troba sobre l'el·lipse i forma un certa angle en graus respecte el semieix major.
   *
   *	Paràmetres d'entrada:
   *		double angle: angle en graus d'un punt sobre l'el·lipse respecte el semieix major a
   *		double a: semieix major
   *		double b: semieix menor
   *	Valor de retorn (double): radi de l'el·lipse en el punt en el qual es coneix l'angle
   **************************************************************************************/
  double c,exc_quadrat;
  c=cos(angle*M_PI/180);
  exc_quadrat=1-b*b/(a*a);
  return (b/sqrt(1-exc_quadrat*c*c));
}

void espera_OK(){
  do{
    do{
      digitalWrite(led,HIGH);  
    }
    while(Serial.available()==0);
    digitalWrite(led,LOW); 
  }	
  while(Serial.read()!='K'); //Les respostes finalitzen amb OK
}

void executa(char peticioAT[])
/*******************************************************************************************
 *	 Es realitza una petició AT i es col·loca la resposta en una cadena de caràcters.
 *
 *	 Paràmetres d'entrada:
 *		char peticióAT[]: apuntador a la cadena de caràcters que conté la comanda AT.
 *******************************************************************************************/
{  
  while (Serial.available()) Serial.read();
  Serial.println(peticioAT); 
  espera_OK();
}	

int demana(char peticioAT[], char data[])
/*******************************************************************************************
 *	 Es realitza una petició AT i es col·loca la resposta en una cadena de caràcters.
 *
 *	 Paràmetres d'entrada:
 *		char peticióAT[]: apuntador a la cadena de caràcters que conté la comanda AT.
 *		char data[]: apuntador a la cadena de caràcters on s'emmagatzemarà la resposta de la 
 *			petició AT.
 *	Valor de retorn (int): 0 si el buffer no data[] no és suficient, 1 si la resposta de la
 *                comanda acaba en OK
 *******************************************************************************************/
{
  int x;

  delay (1500); //Importantíssim. No s'ha d'eliminar.
  /* Entre peticions s'ha de deixar un temps de marge, per tal que el mòdul 3G finalitzi
   operacions. Si aquest temps es treu, s'executa la comanda AT+CGPSINFO moltes més vegades
   i s'encalla en la comanda AT+CHTTPACT. Aquest marge de 1500 ms funciona molt bé.*/
  while (Serial.available()) Serial.read();
  Serial.println(peticioAT); 
  x=0;
  do{
    do{
      digitalWrite(led,HIGH);  
    }
    while(Serial.available()==0);
    digitalWrite(led,LOW);
    data[x]=Serial.read();  
    x++;
    if (x==N_BUFFER) return 0;
  }	
  while(data[x-1]!='K'); //Les respostes finalitzen amb OK
  return 1;
}

void extreu_digits_completant_amb_zeros_per_la_esquerra(int num_digits, char origen[],  char desti[])
/*****************************************************************************************************
 *	Col·loca els dígits de l'origen al destí. Els dígits es copien de dreta a esquerra i es completa amb
 *	zeros fins arribar al nombre total de dígits donat per num_digits.
 *
 *	Paràmetres d'entrada:
 *		int num_digits: indica el nombre total de dígits que ha de contenir el destí
 *		char origen[]: és un apuntador al primer dígit a copiar. Aquest dígit ocupa la posició més a la
 *			dreta.
 *		char desti[]: és un apuntador que conté l'adreça on es copiarà el primer dígit. Els altres 
 *			dígits es copiaran anant cap a l'esquerra. En el cas que el nombre de dígits a copiar sigui 
 *			inferior a num_digits, s'omplirà amb zeros.
 *	Valor de retorn: no retorna res.
 *****************************************************************************************************/
{
  int j=0; /*comptador de dígits extrets*/

  do{
    desti[j]=origen[j];
    j--;
  }
  while (origen[j]>='0'&&origen[j]<='9');
  /*En el cas que el nombre de dígits de l'origen sigui inferior a num_digits es completa amb zeros per l'esquerra.*/
  while (-j<num_digits){
    desti[j]='0';
    j--;	
  } 
}	


void tractament_IMSI(char data[], char json[], int pos_IMSI)
/***************************************************************************************
 * Extreu d'una cadena de caràcters que conté el número IMSI, aquest nombre i el col·loca a
 * la cadena json. El dígit de menys pes de l'IMSI quedarà en la posició indicada per pos_IMSI. En 
 * el cas que el número IMSI tingui menys de 15 dígits, s'omplirà amb zeros per l'esquerra.
 *
 *	Paràmetres d'entrada:
 *		char data[]: És la cadena de caràcters que conté el número IMSI.
 *		char json[]: És la cadena de caràcters on s'ha de col·locar el número IMSI.
 *		int pos_IMSI: Indica la posició en la cadena json que contindrà l'últim dígit de l'IMSI.
 *      Valor que retorna: No reotrna cap valor.
 ****************************************************************************************/
{
  int x;
  x=0; /*índex data	*/

  while(data[x]!='K') x++; /*Avança fins el final de resposta*/
  while(data[x]<'0'||data[x]>'9') x--; /*Retrocedeix fins l'últim dígit de la resposta*/
  extreu_digits_completant_amb_zeros_per_la_esquerra(15, data+ x, json + pos_IMSI);
}

void extreu_bateria(char data[],  struct registre *reg_actual){
  /*********************************************************************************
   * Extreu d'una cadena de caràcters que conté informació relacionada amb la bateria
   * la informació corresponent al percentatge de bateria restant. Emmagatzema aquesta
   * informació mitjançant un apuntador a una estructura del tipus registre.
   *	 
   *	Paràmetres d'entrada:
   *		char data[]: Cadena de caràcters que conté, entre altres coses, el percentatge 
   *			de bateria restant.
   *		registre *reg_actual: Apuntador a una estructura de tipus registre, en la qual
   *			es pot emmagatzemar el percentatge de bateria restant.
   *	Valor de retorn: No retorna res.
   ***********************************************************************************/
  /*Després de demanar l'estat de la bateria amb la comanda AT+CBC, 
   		es rep una resposta del tipus    +CBC=0,75,3.810V
   		75 fa referencia al percentatge de bateria restant de 0 a 100.   */

  int x;
  x=0; //índex data  
  while(data[x]!=',') x++; //Avança fins la primera coma
  x++;
  while(data[x]!=',') x++; //Avança fins la segona coma
  x--; //Retrocedeix a l'ultim dígit del percentatge de bateria restant
  extreu_digits_completant_amb_zeros_per_la_esquerra(3, data + x, (reg_actual->bateria)+2);		
}

double expressa_en_graus(char GPS_angle[])
/**********************************************************************************
 *  Transforma un angle en format ddmm.mmmmmm o bé dddmm.mmmmmm emmagatzemat en una 
 *  cadena de caràcters, a un nombre decimal amb precisió doble.
 *
 *	Paràmetres d'entrada:
 *		char GPS_angle[]: és una cadena de caràcters que conté un angle en graus i minuts.
 *	Valor de retorn (double): Retorna el valor de l'angle en graus. 
 ***********************************************************************************/
{
  int x=0;
  int i_decimals=0;
  double angle,z;
  angle=0;
  do{ /* Es tracten els graus */
    angle*=10;					
    angle+=GPS_angle[x]-0x30; 
    x++;
  } 
  while(GPS_angle[x+2]!='.'); /*ddmm.mmmmmm o bé dddmm.mmmmmm*/
  z=0; /*Es tracten els minuts */
  do{
    z*=10;
    z+=GPS_angle[x]-0x30;	    		
    x++;
  }
  while (GPS_angle[x]!='.');    
  x++; /*Es salta el punt decimal dels minuts */
  do{
    z*=10;
    z+=GPS_angle[x]-0x30;	    		
    x++;
    i_decimals++;
  }
  while (GPS_angle[x]!='\0'); 
  for(x=0;x<i_decimals;x++) z*=0.1;
  z/=60; /*Els minuts  es passen a graus.*/
  angle+=z;
  return angle;
}
void converteix_double_a_cadena_de_caracters(double z, char desti[], int num_digits_enters, int num_digits_decimals){
  /*********************************************************************************************
   *	Converteix un nombre decimal de doble precisió a cadena de caràcters. S'especifiquen el 
   *    nombre de dígits abans i després de la coma. El caràcter corresponent al dígit de menys pes
   *	és col·locarà a desti[0]. El següent dígit amb pes es col·locarà a desti[-1] i així 
   *	successivament.
   *
   *	Paràmetres d'entrada:
   *		double a: nombre decimal de precisió doble que es vol convertir a cadena de caràcters.  
   *			Condició: a és positiu.
   *		char desti[]: apuntador al caràcter obtingut corresponent al dígit de menys pes.
   *		int num_digits_enters: nombre de dígits abans de la coma decimal.
   *		int num_digits_decimals: nombre de dígits després de la coma decimal. 
   *	Valor de retorn: 	No retorna res.
   **********************************************************************************************/
  int k, i_desti;
  char digit;
  double e; //part entera
  e=floor(z);
  z=z-e; //part decimal
  i_desti=-num_digits_decimals+1; //posició primer decimal
  for (k=0;k<num_digits_decimals;k++)
  {
    z*=10;
    digit= ((char)(floor(z)));
    z-=floor(z);
    desti[i_desti]=digit+0x30;
    i_desti++;
  } 
  i_desti=-num_digits_decimals; //posició coma decimal
  desti[i_desti]='.';
  i_desti--;
  for (k=0;k<num_digits_enters;k++){
    digit= (char) (e -floor(e/10)*10);
    e=(e-digit)/10;
    desti[i_desti]=digit+0x30;
    i_desti--;
  }
}

int extreu_subdata(char data[], int posicio, char subdata[], char separador){
  /******************************************************************************
   *  Extreu d'una cadena de caràcters data, els caràcters que hi ha fins un cert 
   *  separador i els col·loca a la cadena de caràcters subdata.
   *
   *  Paràmetres d'entrada:
   * 	char data[]: cadena de caràcters de la qual s'han d'extreure els caràcters, 
   * 		a partir de la posició inicial indicada pel paràmetre posició, fins trobar 
   * 		el caràcter indicat pel caràcter separador.
   * 	int posicio: índex de la posició inicial de la cadena data.
   * 	char subdata[]: cadena on es guardarà el resultat de l'extracció.
   * 	char separador: caràcter en la cadena data que indica que ja s'han extret 
   * 		totes les dates.
   *  Valor de retorn (int): Retorna un enter que indica la posició del caràcter 
   *		després del separador.
   *******************************************************************************/
  int k=0;
  while(data[posicio]!=separador){
    subdata[k]=data[posicio];
    k++;
    posicio++;
  }
  posicio++; /*Salta el separador*/
  return posicio;
}

int extreu_subdata_n(char data[], int posicio, char subdata[], int n){
  /******************************************************************************
   *	Extreu d'una cadena de caràcters data n caràcters  i els col·loca a la 
   *	cadena de caràcters subdata.
   *
   *  Paràmetres d'entrada:
   * 	char data[]: cadena de caràcters de la qual s'han d'extreure els caràcters, 
   * 		a partir de la posició inicial indicada pel paràmetre posició, fins trobar 
   * 		el caràcter indicat pel caràcter separador.
   * 	int posicio: índex de la posició inicial de la cadena data.
   * 	char subdata[]: cadena on es guardarà el resultat de l'extracció.
   * 	int n: nombre de caràcters a extreure.
   *   Valor de retorn (int): Retorna un enter que indica la posició del caràcter següent a 
   *		a l'últim caràcter extret en la cadena data.
   *******************************************************************************/
  int k=0;
  while(k<n){
    subdata[k]=data[posicio];
    k++;
    posicio++;
  }
  return posicio;
}

int extreu_subdades_GPS(char data[], struct registre *reg_actual){
  /**************************************************************************
   *	Extreu informació d'una cadena de caràcters amb informació GPS i la 
   *	col·loca a una estructura registre.
   *
   *	Paràmetres d'entrada:
   *		char data[]: cadena de caràcters amb la informació GPS
   *		registre *reg_actual: apuntador a una estructura registre on
   *			s'emmagatzemaran les dades recollides pel GPS		
   *	Valor de retorn (int):
   *		0: si la informació del GPS no està disponible.
   *		1: altrament
   **************************************************************************/
  int x;
  char latitud[]="ddmm.mmmmmm";
  char longitud[]="dddmm.mmmmmm";
  char orientacio[]="N";

  x=0;
  while(data[x]!=':') x++;
  if(data[x+1]==',') return 0; 
  /*Informació del GPS no disponible. La resposta és del tipus: 
   		+CGPSINFO:,,,,,,,,  */

  else /*La reposta és del tipus: 
   	+CGPSINFO:0912.507634,S,031238.783121,E,060712,201820.0,1519.0,0,0*/
  {
    x++;
    while (data[x]==' ') x++; 
    x=extreu_subdata( data, x, latitud,',');
    reg_actual->latitud=expressa_en_graus(latitud);	

    x=extreu_subdata( data, x, orientacio,',');
    if (orientacio[0]=='S') reg_actual->latitud=-(reg_actual->latitud);

    x=extreu_subdata( data, x, longitud,',');
    reg_actual->longitud=expressa_en_graus(longitud);

    x=extreu_subdata( data, x, orientacio,',');		
    if (orientacio[0]=='W') reg_actual->longitud=-(reg_actual->longitud);

    x=extreu_subdata_n( data, x, reg_actual->dia,2);
    x=extreu_subdata_n( data, x, reg_actual->mes,2);
    x=extreu_subdata( data, x, reg_actual->any,',');
    x=extreu_subdata_n( data, x, reg_actual->hora,2);
    x=extreu_subdata_n( data, x, reg_actual->minut,2);
    x=extreu_subdata_n( data, x, reg_actual->segon,2);
  }
  return 1;	
}
void copia_caracters(int num_caracters, char origen[], char desti[])
/*******************************************************************************
 *	Copia un nombre de caràcters igual a num_caracter d'origen a desti.
 *
 *	Paràmetres d'entrada:
 *		int num_caracters: nombre de caràcters que es volen copiar.
 *		char origen[]: Apuntador a la cadena de caràcters a copiar.
 *		char desti[]: adreça on es volen copiar els caràcters
 *	Valor de retorn: no retorna cap valor.
 ********************************************************************************/
{
  int i;
  for (i=0; i<num_caracters; i++) desti[i]=origen[i];
}

void punts_insercio(char json[], struct posicions *pos){
  /*********************************************************************************
   *	Busca, en la cadena json, els punts d'inserció de: latitud, signe de la latitud,
   *	longitud, signe de la longitud, IMSI, bateria, date, time. Aquestes 
   *	posicions quedaran emmagatzemades en una estructura del tipus posicions, de la qual 
   *	es passa, com a paràmetre d'entrada, l'adreça de memòria on està emmagatzemada.
   *
   *	Paràmetres d'entrada:
   *		char json[]: cadena de caràcters json.
   *		struct posicions *pos: apuntador a una estructura posicions.
   *	Valor de retorn: no retorna cap valor.
   *********************************************************************************/
  int k;
  k=0;
  while (!(json[k]=='s'&&json[k+1]=='g')) k++;
  pos->signe_longitud=k;
  while (!(json[k-1]=='g'&&json[k]=='g'&&json[k+1]=='%')) k++;
  pos->longitud=k;
  while (!(json[k]=='s'&&json[k+1]=='g')) k++;
  pos->signe_latitud =k;
  while (!(json[k-1]=='g'&&json[k]=='g'&&json[k+1]=='%')) k++;
  pos->latitud =k;
  while (!(json[k-1]=='x'&&json[k]=='x'&&json[k+1]=='%')) k++;
  pos->IMSI=k;
  while (!(json[k]=='p'&&json[k+1]=='p'&&json[k+2]=='p')) k++;
  pos->bateria=k;		
  while (!(json[k]=='y'&&json[k+1]=='y'&&json[k+2]=='-')) k++;
  pos->any=k;		
  while (!(json[k]=='m'&&json[k+1]=='m')) k++;
  pos->mes=k;	
  while (!(json[k]=='d'&&json[k+1]=='d')) k++;
  pos->dia=k;	
  while (!(json[k]=='h'&&json[k+1]=='h')) k++;
  pos->hora=k;	
  while (!(json[k]=='m'&&json[k+1]=='m')) k++;
  pos->minut=k;	
  while (!(json[k]=='s'&&json[k+1]=='s')) k++;
  pos->segon=k;
  return;
}

void inserta_json(struct registre *reg_actual, char json[], struct posicions pos)
/************************************************************************************
 *	Inserta en la cadena json, en les posicions donades per pos, les dades emmagatzemades 
 *	en reg_actual.
 *
 *	Paràmetres d'entrada:
 *		registre *reg_actual: conté l'estat de la bateria i la informació recollida 
 *			mitjançant el GPS.
 *		char json[]: cadena de caràcters on s'ha d'inserir la informació recollida.
 *		posicions pos: estructura que indica les diferents posicions en la cadena json, on
 *			s'ha d'inserir la informació recollida.
 *	Valor de retorn: no retorna res.	
 *************************************************************************************/
{
  if ((reg_actual->latitud)>0){
    json[pos.signe_latitud]='+';					    
    converteix_double_a_cadena_de_caracters(reg_actual->latitud, json+ pos.latitud, NUM_DIGITS_ENTERS_LATITUD, NUM_DIGITS_DECIMALS_LATITUD);
  }  
  else{
    json[pos.signe_latitud]='-';
    converteix_double_a_cadena_de_caracters(-(reg_actual->latitud), json+ pos.latitud, NUM_DIGITS_ENTERS_LATITUD, NUM_DIGITS_DECIMALS_LATITUD);
  }
  if ((reg_actual->longitud)>0){
    json[pos.signe_longitud]='+';					    
    converteix_double_a_cadena_de_caracters(reg_actual->longitud, json+ pos.longitud, NUM_DIGITS_ENTERS_LONGITUD, NUM_DIGITS_DECIMALS_LONGITUD);
  }  
  else{
    json[pos.signe_longitud]='-';
    converteix_double_a_cadena_de_caracters(-(reg_actual->longitud), json+ pos.longitud, NUM_DIGITS_ENTERS_LONGITUD, NUM_DIGITS_DECIMALS_LONGITUD);
  }

  copia_caracters(2, reg_actual->any, json+pos.any);
  copia_caracters(2, reg_actual->mes, json+pos.mes);		
  copia_caracters(2, reg_actual->dia, json+pos.dia);		
  copia_caracters(2, reg_actual->hora, json+pos.hora);		
  copia_caracters(2, reg_actual->minut, json+pos.minut);		
  copia_caracters(2, reg_actual->segon, json+pos.segon);
  copia_caracters(N_BATERIA, reg_actual->bateria, json+pos.bateria);		
}

void envia_peticio_HTTP(  char json[], char json_cont[], int Content_Length){
  /************************************************************************************
  * Envia pel pin TX  la capçalera HHTP i les dades HTTP.
  *************************************************************************************/
  Serial.println("POST /arcgis/rest/services/TrailWalker/FeatureServer/0/addFeatures HTTP/1.1");
  Serial.print("Host: ");
  Serial.print(server);
  Serial.print(':');
  Serial.println(port);
  Serial.print("Content-Length: ");
  Serial.println(Content_Length);
  Serial.print("Origin: http://");
  Serial.print(server);
  Serial.print(':');
  Serial.println(port);

  Serial.println("Content-Type: application/x-www-form-urlencoded");
  Serial.println("Accept: text/html");
  Serial.print("Referer: http://");
  Serial.print(server);
  Serial.print(":");
  Serial.print(port);	
  Serial.println("/arcgis/rest/services/TrailWalker/FeatureServer/0/addFeatures");
  Serial.println("Accept-Charset: utf-8");
  //Abans d'enviar dades 
  Serial.write(0x0D);       //caràcter de retorn de carro
  Serial.write(0x0A);       //caràcter d'avançament de línia

  Serial.print(json);
  Serial.print(Equip);
  Serial.print(json_cont);

  Serial.write(0x1A);       //Indica final de las dades enviades
  Serial.write(0x0D);       //caràcter de retorn de carro
  Serial.write(0x0A);       //caràcter d'avançament de línia
  Serial.flush();
}

int senyal(char data[]){
  /*********************************************************************************
   *  Retorna un indicador de la força del senyal.
   *   Parametres d'entrada: 
   *      char data[]: adreça de memòria, que es farà servir com a búffer.
   *   Valor de retorn (int):  de 0 a 31 va de menys a més. 99 significa
   *     valor desconegut o no detectable.
   **********************************************************************************/
  int sq=0; 
  int x=0; 
  demana ("AT+CSQ",data);
  while (data[x]!=',') x++;

  x--;
  sq=data[x]-0x30;
  x--;
  if (data[x]!=' ') sq+=(data[x]-0x30)*10;
  return sq;
}


int envia_POST( char json[], char json_cont[], int Content_Length, char data1[], int M_BUFFER1){
  /******************************************************************************************
   *	Envia dades mitjançant el protocol HTTP amb un petició POST. Les dades que conté aquesta
   *	petició és la cadena de caràcters json.
   *
   *	Paràmetres d'entrada:
   *		char json[]: aquesta cadena de caràcters són les dades que transportarà la petició POST.
   *	Valor de retorn (int): 
   *
   *		1: La transmissió s'ha realitzat correctament.
   *		0: Senyal insuficient, error en establir la connexió o no s'ha creat l'objecte en el servidor.
   *            2: S'ha establert la connexió. Però el mòdul s'ha quedat penjat en la transmissió o recepció 
   *               de resposta. Es reinicia el mòdul. 
   *******************************************************************************************/
  char data[M_BUFFER];
  int x;
  int cond, cond1;
  float k,j;
  int sq=0;   
  sq=senyal(data);
  Serial.println(sq);
  if (sq<2||sq==99)return 0; // De 0 a 31 va de menys senyal a més senyal. 99 significa valor desconegut o no detectable.
  //  No s'han fet prou proves per detectar el funcionament del codi quan la senyal és insuficient.
  // Aquesta és una mesura de control per evitar errors.

  Serial.flush();
  while(Serial.available()) Serial.read();         
  Serial.print("AT+CHTTPACT=\""); //Connecta amb el servidor HTTP 
  Serial.print(server);
  Serial.print("\",");
  Serial.println(port); 


  k=0;
  for(x=0;x<M_BUFFER;x++) data[x]=0;
  x=0;
  do{
    do{
      digitalWrite(led,HIGH); 
      k++;
      if (k>10000000) break; 
      /*Aquest valor és unes 40 vegades el que sol trigar.
       Es tracta d'evitar un bucle infinit en cas d'error */
    }
    while(Serial.available()==0);
    if (k>10000000) break; 

    digitalWrite(led,LOW);
    data[x]=Serial.read();
    cond= transmissio_correcta(data,  x, M_BUFFER);  
    if (cond==ERROR){
      // Quan la resposta és un codi d'error (220,221,....,227), el mòdul es queda en un estat que no accepta més 
      // comandes AT. La única possibilitat trobada és apagar el mòdul.

      reiniciar_modul_bloquejat();
      return ESTAT_ERROR;
    }      

    x=(x+1)%M_BUFFER;  
  }
  while(cond!=ESTAT_REQUEST); /* hi ha més dades per llegir*/


  envia_peticio_HTTP( json, json_cont, Content_Length);         

  for(x=0;x<M_BUFFER1;x++) data1[x]=0; 
  x=0;
  j=0;
  do{
    do{
      digitalWrite(led,HIGH);
      j++;
      if (j>1000000) break; 
      /*Aquest valor és unes 40 vegades el que sol trigar.
       Es tracta d'evitar un bucle infinit en cas d'error */
    }
    while(Serial.available()==0);
    if (j>1000000) break;   
    digitalWrite(led,LOW);
    data1[x]=Serial.read();
    cond1= transmissio_correcta(data1,  x, M_BUFFER1);
    x=(x+1)%M_BUFFER1;  
  }
  while(cond1==ESTAT_REBENT); /* hi ha més dades per llegir*/

  if (cond1==ESTAT_REBENT){//Representa que s'ha sortit del bucle perquè trigava més del valor esperat
    reiniciar_modul();
    return ESTAT_REINICIAT; 
  }  

/*Informació per depurar. Es fan servir 2 búffers diferents per depurar les dues respostes
  Serial.print("k=");
  Serial.print(k);
  Serial.print(", j=");
  Serial.println(j);
  visualitza_data(data,M_BUFFER);
  Serial.println("mes dades: ");
  visualitza_data(data1,M_BUFFER1);
  Serial.print("Condicio: "); 
  Serial.print(cond); 
  Serial.print("    Condicio: "); 
  Serial.println(cond1);
 */
  return cond1; 
}

void visualitza_data(char data[], int mida){  
/* Funció per depurar. El codi no utilitza aquesta funció*/  
  int x;

  for(x=0;x<mida;x++){
    if (data[x]=='+') Serial.print(' ');
    else if (data[x]>0x29) Serial.print (data[x]); 
    else if (data[x]==0x0A) Serial.write(0x0A);          
    else if (data[x]==0x0D) Serial.write(0x0D);
    else Serial.print(' ');
  }
  Serial.println("");
}

void visualitza_resposta(int espera_milisegons){ 
/* Funció per depurar. El codi no utilitza aquesta funció*/  
  char c;
  delay(espera_milisegons);
  while(Serial.available()){
    c=Serial.read(); 
    if (c=='+') Serial.print(' '); 
    else Serial.print(c);
  };
  Serial.println("");
}
int transmissio_correcta(char data[],  int i, int i_max){
  /****************************************************************************************************
   * Donat el búffer data amb capacitat i_max caràcters, on s'emmagatzemen les dades rebudes 
   * d'un resposta, i donada una posició actual i en el búffer, es tracta d'esbrinar si la resposta 
   * rebuda significa que la transmissió GPRS s'ha realitzat correctament (retorna 1), 
   * si s'ha produït un error (retorna 0) o si s'ha de continuar analitzant la informació rebuda
   * (retorna 2).
   * La resposta que es busca és una de les següents:
   *	+CHTTPACT: 0 --->La transmissió GPRS s'ha realitzat correctament. Hi ha dos casos:
   *            1) Prèviament s'ha detectat la creació d'un objecte amb identificador objectID. 
   *            2) No s'ha detectat la creació d'aquest objecte.
   *	+CHTTPACT: DATA --->Hi ha més dades a rebre de resposta.
   *    +CHTTPACT: 220 (221, 222, ..., 227)
   *	+CHTTPACT: <altres> ---> La transmissió GPRS no s'ha realitzat correctament, hi ha hagut
   *				un error. El valor d'<altres> pot ser ERROR o un codi d'error.
   *
   * Paràmetres d'entrada:
   *	char data[]: búffer que emmagatzema les dades rebudes
   *	int i: posició actual dins el búffer.
   *	int i_max: grandària del búffer.
   * Valor de retorn (int):
   *       
   *	1: La transmissió GPRS s'ha realitzat correctament i s'ha creat l'objecte amb identificador 
   *          objectId.
   *	2: Encara hi ha dades per rebre.
   *    3: S'ha rebut REQUEST.
   *    4: S'ha rebut produït un error del tipus 220, 221, ..., 227.
   *	0: S'ha produït un altres tipus d'error. 
   *          
   ****************************************************************************************************/
  char c;
  static int objectId=0;
  if (data[(i-4+i_max)%i_max]=='C'){

    if (data[(i-3+i_max)%i_max]=='T'){

      if (data[(i-2+i_max)%i_max]==':'){
        if (data[(i-1+i_max)%i_max]==' '){
          c=data[i%i_max];
          if (c=='R') return 3; /* Cas CHTTPACT: RESQUEST*/
          else if (c=='2') return 4; // codi d'error: 220, 221,..., 227
          else if (c=='0'){
            if (objectId==1){
              objectId=0; 
              return 1; /*Transmissió correcta.
               Objecte creat correctament en el servidor. Resposta rebuda correctament. 
               No hi  ha més dades per rebre*/
            }
            else { 
              return 0; /*S'ha rebut una resposta però no s'ha creat l'objecte en el servidor. Per exemple,
               el servidor pot estar momemtàniament fora de servei*/
            }
          }  

          else if (c=='D') return 2; /*Hi ha mes dades per rebre */
          else return 0; /*S'ha produït algun altre tipus d'error.*/


        }
      }
    }
  }
  if (data[(i-4+i_max)%i_max]=='e'){

    if (data[(i-3+i_max)%i_max]=='c'){

      if (data[(i-2+i_max)%i_max]=='t'){
        if (data[(i-1+i_max)%i_max]=='I'){
          if (data[i%i_max]=='d')  objectId=1;				
        }
      }
    }
  }

  return 2; /*Hi ha més dades per rebre */
}	


int longitud_contingut( char *s1, char *s2, char*s3){
  /******************************************************************************
   *	Retorna el nombre de caràcters total de les tres cadenes
   *		char *s1: cadena de caràcters que acaba amb '\0'.
   *		char *s2: cadena de caràcters que acaba amb '\0'.
   *		char *s3: cadena de caràcters que acaba amb '\0'.
   *	Valor de retorn (int): Retorna el nombre de caràcters total.
   ********************************************************************************/
  int i, j;
  i=0;
  j=0;   	
  while (s1[i]!='\0'){
    i++; 
    j++; 
  }
  i=0;  	
  while (s2[i]!='\0'){
    i++; 
    j++; 
  }
  i=0;  	
  while (s3[i]!='\0') {
    i++; 
    j++; 
  }
  return j;
}


void loop(){
  /***********************************************************************
   *	Aquesta funció s'executa infinitament. És la part principal del programa.
   *	Conté el codi perquè el microntrolador l'executi de manera recursiva, 
   *	la placa d'Arduino tingui energia.
   *************************************************************************/


  int k;	
  char data[N_BUFFER];
  static double r; /* radi de la terra en la latitud de la meta */

  static struct registre reg_guardat[MEM_MAX]; 
  struct registre reg_actual[1];
  static int i_guardar=0; //indica quin serà el següent registre a escriure, del vector reg_guardat. 
  static int per_enviar=0; //Inicialment no hi ha dades per enviar
  int pots_enviar;

  static int  Content_Length; 
  static	char json[]=
    "features=%5B%7B%22geometry%22%3A%7B%22x%22%3Asggg.gggggggggg%2C%22y%22%3Asgg.gggggggggg%2C%22spatialReference%22%3A%7B%22wkid%22%3A104910%7D%7D%2C%22attributes%22%3A%7B%22CodiEquip%22%3Axxxxxxxxxxxxxxx%2C%22Bateria%22%3Appp%2C%22Entrada%22%3A+%2220yy-mm-dd+hh%3Amm%3Ass%22%2C%22Tipus%22%3A%22Arduino%22%2C%22Equip%22%3A";

  char json_cont[]="%7D%7D%5D&gdbVersion=&rollbackOnFailure=true&f=html";
  /******************************************************************************************
   * 	Equivalència:
   * 					%5B [
   * 					%5D ]
   * 					%7B {
   * 					%7D }
   * 					%2C ,
   * 					%3A :
   * 					%22 "
   * 	Per exemple: el caràcter [ té un codi ASCII 0x5b. Per tant, en un llenguatge més entenedor:
   * features=[{"geometry":{"x":2.894726,"y":41.827171,"spatialReference":{"wkid":104910}},"attributes":{"CodiEquip":xxxxxxxxxxxxxxx,"Bateria":100,"Entrada": "2012-10-22 10:53:12","Tipus":"Arduino","Equip":ARDUINO_000}}]&gdbVersion=&rollbackOnFailure=true&f=html
   *******************************************************************************************/
  static int primera_vegada=1;
  static posicions pos;
  int estat_enviament;


  if (primera_vegada){
    punts_insercio(json, &pos);
    demana("AT+CIMI",data);/*demana l'IMSI: International Mobil Subscriber Identifier*/

    tractament_IMSI(data, json, pos.IMSI); /*inserta l'IMSI en la cadena json.*/
    Content_Length=longitud_contingut(json,Equip,json_cont); 

    for (k=0;k++;k<MEM_MAX) reg_guardat[k].enviat=1; 
    /* Inicialment, no hi ha cap dada guardada per enviar */

    r=radi_elipse(LATITUD_META, R_EQUATORIAL, R_POLAR);
    primera_vegada=0;
  }
  //___________________________________________________________________________________________
  //___________________________       GPS         _____________________________________________
  //___________________________________________________________________________________________

  executa("AT+CGPS=1,1");
  do{
    while(!demana("AT+CGPSINFO", data)); // Existeix la possibilitat que el buffer data[] no fos suficient, 
    // si es donés el cas d'un funcionament inadequat del GPS.  S'executaria la comanda fins que el GPS no 
    // fes aquest error, si és que el fa alguna vegada.    
  }
  while(!extreu_subdades_GPS(data, reg_actual)); 
  //demana informació GPS mentre no es pugui extreure informació perquè la resposta no conté encara dades.



  /*
  //__________________________________________________________________________________________________________________
  // S'ha detectat un fucionament incorrecte del mòdul GPS, després de portar molta estona encès: les dades que torna la comanda 
  // AT+CPSINFO no són noves, són les anteriors. El GPS no funciona correctament, no adquireix dades noves. De totes maneres, sembla 
  // que la raó és que des d'on es realizen, les proves (interior) no hi ha visibilitat de prous satel·lits durant un cert interval
  // de temps. En algun cas ha estat un interval d'una hora i mitja, en algun cas ha estat un interval de més de 4 hores.
  // Es pot prescindir d'aquest tros de codi. Aquest fragment de codi funciona correctamnet.
  
  static temps t;
  if (reg_actual->segon[1]==t.segon[1]){
    if (reg_actual->segon[0]==t.segon[0]){
      if (reg_actual->minut[1]==t.minut[1]){
        if (reg_actual->minut[0]==t.minut[0]){
          if (reg_actual->hora[1]==t.hora[1]){
            if (reg_actual->hora[0]==t.hora[0]){
              reiniciar_modul();
              return; //Es torna a repetir el loop 
            }
          }
        }                 
      } 
    }
  }   
  // En el cas d'arribar a aquest punt significa que les dades que adquirit el GPS són noves.  Es guarden aquestes
  // noves dades per comparar en la següent iteració
  t.segon[1]=reg_actual->segon[1];
  t.segon[0]=reg_actual->segon[0];
  t.minut[1]=reg_actual->minut[1];
  t.minut[0]=reg_actual->minut[0];
  t.hora[1]=reg_actual->hora[1];
  t.hora[0]=reg_actual->hora[0];  
  //___________________FI del tros de codi del qual es pot prescindir_____________________________________________ 
  //______________________________________________________________________________________________________________ 
*/


  executa("AT+CGPS=0");

  //visualitza_data(data, N_BUFFER); //En el cas que es necessiti depurar


  //___________________________________________________________________________________________
  //___________________________   FI    GPS       _____________________________________________
  //___________________________________________________________________________________________



  demana("AT+CBC", data);/*demana estat bateria*/
  extreu_bateria(data,  reg_actual);
  inserta_json(reg_actual, json, pos);
  estat_enviament=envia_POST(json,json_cont, Content_Length, data, N_BUFFER);
  
  if(estat_enviament==ESTAT_REINICIAT) return;
  
  switchModule(); //apaga el mòdul

  //CAS 1: no s'ha pogut enviar el paquet_____________________________________________________________________
  if (estat_enviament==0){
    per_enviar=1; /* Hi ha paquests per enviar*/
    i_guardar=guarda(reg_actual[0], reg_guardat, i_guardar, MEM_MAX);	
    delay(MINUTS_NO_GPRS*60000); 

  }

  // CAS 2: El paquet s'ha enviat_____________________________________________________________________________
  else if (estat_enviament==1){ 
    if (per_enviar){
      for (k=0;k<MEM_MAX;k++){
        if (reg_guardat[k].enviat==0)
        {
          inserta_json(&(reg_guardat[k]), json, pos);
          executa("AT+CGPS=1,1");
          executa("AT+CGPS=0");
          pots_enviar=envia_POST(json, json_cont, Content_Length, data, N_BUFFER);
          if (pots_enviar) reg_guardat[k].enviat=1;
          else break; 
        }
      }
    }
    if (pots_enviar) per_enviar=0; /* S'han enviat tot els paquets */
    if (DETECTAR_META){
      k=detectar_meta(r, reg_actual->latitud, reg_actual->longitud, LATITUD_META,LONGITUD_META);
      if (k==3) delay(MINUTS*60000); // Queda més de 5 km. 
      else if (k==2) delay(MINUTS*12000); //Queda més d'1 km i com a molt 5 km.
      else if (k==1) delay(MINUTS*1000); //Queda com a molt 1 km.
      else{                  //Queden en  metres una distància inferior o igual a DISTANCIA_META: no s'envien més dades.
        while (1) delay(300000); 
      }
    }
    else delay(MINUTS*60000);
  }
  iniciar_modul();//S'engega el mòdul de nou
}


